unit Collision;

interface

uses
  vectors; // bentigt fr Vektorrechnung

type
  // Array mit den Vertices des Polygons
  TV2fArray = array of TVector2f;
  // Polygon Klasse
  TPolygon = class
  private
    fposition: TVector2f;                             // Position
    fvertices: TV2fArray;                             // Vertices (Objektkoordinaten)
    function GetVertex(n: integer): TVector2f;        // Liefert die Objektkoordinaten
    function GetVertexAbs(n: integer): TVector2f;     // Liefert die absoluten Koordinaten
    procedure SetVertex(n: integer; Value: TVector2f);// Setzt die Objektkoordinaten
    function GetCount: integer;                       // Liefert length(fvertices)
  public
    procedure AddVertex(v: TVector2f);                // Fgt ein Vertex hinzu
    procedure AddVertexAbs(v: TVector2f);             // Fgt ein Vertex mit Weltkoordinaten hinzu
    procedure RemoveVertex(n: integer);               // Entfernt ein Vertex
    property position: TVector2f read fposition write fposition;             // Position
    property vertices[n: integer]: TVector2f read GetVertex write SetVertex; // Vertex Koordinaten
    property vertices_abs[n: integer]: TVector2f read GetVertexAbs;          // Vertex Weltkoordinaten
    property Count: integer read GetCount;                                   // siehe GetCount
  end;
  TPolygonArray = array of TPolygon;
  // Kreis Klasse
  TCircle = class
  private
    fposition: TVector2f;
    fradius: extended;
  public
    property position: TVector2f read fposition write fposition;
    property radius: extended read fradius write fradius;
  end;

// Export Kollisionserkennung
function PolyPolyIntersect(A, B: TPolygon): boolean;
function CircleCircleIntersect(A, B: TCircle): boolean;
function CirclePolyIntersect(P: TPolygon; C: TCircle): boolean;

implementation

procedure TPolygon.AddVertexAbs(v: TVector2f);
begin
  setlength(fvertices, length(fvertices) + 1);
  fvertices[high(fvertices)] := v2f_sub(v, position);
end;

procedure TPolygon.AddVertex(v: TVector2f);
begin
  setlength(fvertices, length(fvertices) + 1);
  fvertices[high(fvertices)] := v;
end;

procedure TPolygon.RemoveVertex(n: integer);
var
  i: integer;
begin
  for i := n to high(fvertices) - 1 do
    fvertices[i] := fvertices[i + 1];
  setlength(fvertices, length(fvertices) - 1);  
end;

function TPolygon.GetVertex(n: integer): TVector2f;
begin
  result := fvertices[n];
end;

function TPolygon.GetVertexAbs(n: integer): TVector2f;
begin
  result := v2f_add(fvertices[n], fposition);
end;

procedure TPolygon.SetVertex(n: integer; Value: TVector2f);
begin
  fvertices[n] := Value;
end;

function TPolygon.GetCount: integer;
begin
  result := length(fvertices);
end;

function PolyPolyIntersect(A, B: TPolygon): boolean;
var
  i, j, l: integer;
  tmp, proj, voffset: TVector2f;
  dp, amin, amax, bmin, bmax, d1, d2, foffset: extended;
begin
  // Offset berechnen
  voffset := v2f_sub(A.position, B.position);
// A - alle Projektionsgeraden ermitteln und projezieren
  for i := 0 to (a.count - 1) do
  begin
    l := i + 1;
    if l > (a.count - 1) then
      l := 0;
    // Berechnung der Seitenflche
    tmp := v2f_sub(a.vertices[l], a.vertices[i]);
    // Berechnet die Normale der Seitenflche
    proj := v2f_normalize(to_v2f(-tmp.y, tmp.x));
    // Projeziert den ersten Wert
    amin := v2f_dotproduct(a.vertices[0], proj);
    amax := amin;
    // Findet den kleinsten und grten projezierten Wert fr die Gerade fr A
    for j := 1 to (a.count - 1) do
    begin
      // projezieren
      dp := v2f_dotproduct(a.vertices[j], proj);
      if dp < amin then
        amin := dp;
      if dp > amax then
        amax := dp;
    end;
    // s.o.
    bmin := v2f_dotproduct(b.vertices[0], proj);
    bmax := bmin;
    // B
    for j := 1 to (b.count - 1) do
    begin
      dp := v2f_dotproduct(b.vertices[j], proj);
      if dp < bmin then
        bmin := dp;
      if dp > bmax then
        bmax := dp;
    end;
    // 1D Kollision
    foffset := v2f_dotproduct(voffset, proj);
    amin := amin + foffset;
    amax := amax + foffset;
    d1 := amin - bmax;
    d2 := bmin - amax;
    // Wenn es keine berschneidung gibt, abbrechen -> keine Kollision
    if (d1 > 0) or (d2 > 0) then
    begin
      result := false;
      exit;
    end;
  end;
// B - alle Projektionsgeraden ermitteln und projezieren (s.o.)
  for i := 0 to (b.count - 1) do
  begin
    l := i + 1;
    if l > (b.count - 1) then
      l := 0;
    tmp := v2f_sub(b.vertices[l], b.vertices[i]);
    proj := v2f_normalize(to_v2f(-tmp.y, tmp.x));
    amin := v2f_dotproduct(a.vertices[0], proj);
    amax := amin;
    for j := 1 to (a.count - 1) do
    begin
      dp := v2f_dotproduct(a.vertices[j], proj);
      if dp < amin then
        amin := dp;
      if dp > amax then
        amax := dp;
    end;
    bmin := v2f_dotproduct(b.vertices[0], proj);
    bmax := bmin;
    for j := 1 to (b.count - 1) do
    begin
      dp := v2f_dotproduct(b.vertices[j], proj);
      if dp < bmin then
        bmin := dp;
      if dp > bmax then
        bmax := dp;
    end;
    foffset := v2f_dotproduct(voffset, proj);
    amin := amin + foffset;
    amax := amax + foffset;
    d1 := amin - bmax;
    d2 := bmin - amax;
    if (d1 > 0) or (d2 > 0) then
    begin
      result := false;
      exit;
    end;
  end;
  // Kollision
  result := true;
end;

function CircleCircleIntersect(A, B: TCircle): boolean;
begin
  result := (sqr(A.position.x - B.position.x) + sqr(A.position.y - B.position.y)) < sqr(A.radius + B.radius);
end;

function CirclePolyIntersect(P: TPolygon; C: TCircle): boolean;
var
  i, j, l: integer;
  tmp, proj, voffset: TVector2f;
  dp, pmin, pmax, cmin, cmax, d1, d2, foffset: extended;
begin
  // Offset berechnen
  voffset := v2f_sub(P.position, C.position);
// P - alle Projektionsgeraden ermitteln und projezieren
  for i := 0 to (P.count - 1) do
  begin
    l := i + 1;
    if l > (P.count - 1) then
      l := 0;
    // Berechnung der Seitenflche
    tmp := v2f_sub(P.vertices[l], P.vertices[i]);
    // Berechnet die Normale der Seitenflche
    proj := v2f_normalize(to_v2f(-tmp.y, tmp.x));
    // Projeziert den ersten Wert
    pmin := v2f_dotproduct(P.vertices[0], proj);
    pmax := pmin;
    // Findet den kleinsten und grten projezierten Wert fr die Gerade fr A
    for j := 1 to (P.count - 1) do
    begin
      // projezieren
      dp := v2f_dotproduct(P.vertices[j], proj);
      if dp < pmin then
        pmin := dp;
      if dp > pmax then
        pmax := dp;
    end;
    cmax := v2f_dotproduct(v2f_scale(proj, C.radius), proj);
    cmin := -cmax;
    // 1D Kollision
    foffset := v2f_dotproduct(voffset, proj);
    pmin := pmin + foffset;
    pmax := pmax + foffset;
    d1 := pmin - cmax;
    d2 := cmin - pmax;
    // Wenn es keine berschneidung gibt, abbrechen -> keine Kollision
    if (d1 > 0) or (d2 > 0) then
    begin
      result := false;
      exit;
    end;
//C - Alle Projektionsgeraden ermitteln und projezieren
    proj := v2f_normalize(v2f_sub(C.position, P.vertices_abs[i]));
    // s.o.
    pmin := v2f_dotproduct(P.vertices[0], proj);
    pmax := pmin;
    for j := 1 to (P.count - 1) do
    begin
      dp := v2f_dotproduct(P.vertices[j], proj);
      if dp < pmin then
        pmin := dp;
      if dp > pmax then
        pmax := dp;
    end;
    cmax := v2f_dotproduct(v2f_scale(proj, C.radius), proj);
    cmin := -cmax;
    foffset := v2f_dotproduct(voffset, proj);
    pmin := pmin + foffset;
    pmax := pmax + foffset;
    d1 := pmin - cmax;
    d2 := cmin - pmax;
    if (d1 > 0) or (d2 > 0) then
    begin
      result := false;
      exit;
    end;
  end;
  result := true;
end;

end.
