(*******************************************************************************
  Author
  ------
  Kai Rhr (Seth)
  email : info@exec-dev.de
  visit: http://www.exec-dev.de | http://www.delphigl.com

  License
  -------
  Copyright (C) 2007 www.delphigl.com

  This license applies to everything on www.delphigl.com, except where otherwise noted.

  This software is provided 'as-is', without any express or implied warranty.  In no event will the authors 
  be held liable for any damages arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose, including commercial applications, 
  and to alter it and redistribute it freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not claim that you wrote the original 
  software. If you use this software in a product, an acknowledgment in the product documentation would be 
  appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Description
  -----------
  Demonstration of the separating axis theorem (collision of a circle and a polygon)   

  Beschreibung
  -------------  
  Demonstration des Separating Axis Theorems (Kollision eines Kreises und eines Polygons)      

*******************************************************************************)

unit UMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Vectors, Collision, ExtCtrls, StdCtrls;

type
  TMainForm = class(TForm)
    Image1: TImage;
    Label1: TLabel;
    procedure Image1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    A, B: TPolygon;
    C: TCircle;
    procedure DrawPolygon(A: TPolygon; aColor: TColor);
    procedure DrawCircle(C: TCircle; aColor: TColor);
    procedure DrawScene;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

procedure TMainForm.DrawPolygon(A: TPolygon; aColor: TColor);
var
  i, l: integer;
begin
  for i := 0 to A.Count - 1 do
  begin
    l := i + 1;
    if l > (A.Count - 1) then
      l := 0;
    with Image1.Canvas do
    begin
      pen.color := aColor;
      MoveTo(round(A.vertices_abs[l].x), round(A.vertices_abs[l].y));
      LineTo(round(A.vertices_abs[i].x), round(A.vertices_abs[i].y));
    end;
  end;
end;

procedure TMainForm.DrawCircle(C: TCircle; aColor: TColor);
begin
  with Image1.Canvas do
  begin
    Pen.Color := aColor;
    Ellipse(round(C.position.x - C.radius), round(C.position.y - C.radius),
                          round(C.position.x + C.radius), round(C.position.y + C.radius));
  end;
end;

procedure TMainForm.DrawScene;
var
  acolor, bcolor, ccolor: TColor;
begin
  image1.Picture.Bitmap := nil;
  image1.Canvas.Pen.color := clblack;
  acolor := clblack;
  bcolor := clblack;
  ccolor := clblack;
  if PolyPolyIntersect(a, b) then
  begin
    acolor := clred;
    bcolor := clred;
  end;
  if CirclePolyIntersect(A, C) then
  begin
    acolor := clred;
    ccolor := clred;
  end;
  if CirclePolyIntersect(B, C) then
  begin
    bcolor := clred;
    ccolor := clred;
  end;
  DrawPolygon(A, acolor);
  DrawPolygon(B, bcolor);
  DrawCircle(C, ccolor);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  doublebuffered := true;
  A := TPolygon.Create;
  B := TPolygon.Create;
  C := TCircle.Create;
  with A do
  begin
    position := to_v2f(100, 100);
    AddVertex(to_v2f(50, -50));
    AddVertex(to_v2f(50, 50));
    AddVertex(to_v2f(-50, 50));
    AddVertex(to_v2f(-50, -50));
  end;
  with B do
  begin
    position := to_v2f(350, 200);
    AddVertex(to_v2f(50, -50));
    AddVertex(to_v2f(25, 25));
    AddVertex(to_v2f(-50, -50));
  end;
  C.position := to_v2f(200, 200);
  C.radius := 50;
  DrawScene;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  A.Free;
  B.Free;
  C.Free;
end;

procedure TMainForm.Image1MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if ssleft in shift then
  begin
    C.position := to_v2f(x, y);
    DrawScene;
  end;  
end;

end.
